/******************************************************************************
 * CONSTANTS                                                                  *
 ******************************************************************************/
#define USAGE "Parameters:\n\t-d TARGET_DIST\n\t-a ANNOTATIONS\n\t-g CONTROL_GENES\n\t-s SAMPLE_SIZE\n\t-m MAX_ITER\n\t-t TOLERANCE\n\t-n NUM_SAMPLES\n\t-o OUTPUT\n\t[-c EXP_COEFF]"
#define GO_SIZE 20
#define GENE_NAME_SIZE 50
#define MAX_STR_LEN 1000
#define READ_BUF 10000

#define min( a, b ) ( ((a) < (b)) ? (a) : (b) )

/******************************************************************************
 * GLOBAL VARIABLES                                                           *
 ******************************************************************************/

gsl_rng *rng;
const gsl_rng_type * T;

/******************************************************************************
 * STRUCTURES                                                                 *
 ******************************************************************************/

typedef struct {
  char gene_name[GENE_NAME_SIZE];
  unsigned int *terms;
  unsigned int num_terms;
} annotation;

typedef struct parameters { /* user-defined parameters */
  char target_file_name[MAX_STR_LEN];
  char ann_file_name[MAX_STR_LEN];
  char control_genes_file_name[MAX_STR_LEN];
  char output_file_name [MAX_STR_LEN];
  double max_iter;
  double tolerance;
  double exp_coeff;
  unsigned int size_sample;
  unsigned int num_samples;
} Parameters;

typedef struct {
  double squared_norm;
  double num_iter;
} sampling_log;

typedef struct {
  char **terms;
  unsigned int *frequency;
  unsigned int num_terms;
} target_distr;

/******************************************************************************
 * PROTOTYPES                                                                 *
 ******************************************************************************/

double calculate_dist(target_distr, unsigned int *);

void calculate_term_distr(unsigned int *, unsigned int *, unsigned int,
			  annotation *);

int read_annotations(char *, annotation **, target_distr);

void read_target_distr(char *, target_distr *);

double read_control_genes(char *, annotation *, unsigned int, unsigned int **);

void recalculate_term_distr(unsigned int *, annotation *, unsigned int,
			    unsigned int);

sampling_log sampler(target_distr, annotation *, unsigned int *,
		     const unsigned int, const unsigned int, double, double,
		     unsigned int*, double);

void set_parameters(int, char **, Parameters *);


